import { Connection, PublicKey, Transaction } from '@solana/web3.js';
import { Jupiter, RouteInfo } from '@jup-ag/core';
import { LiquidityPoolJsonInfo } from '@raydium-io/raydium-sdk';
/**
 * SwapToLP class for swapping SPL tokens to LP tokens through rebalancing and staking.
 */
export declare class SwapToLP {
    /**
     * Properties
     */
    connection: Connection;
    jupiter: Jupiter;
    /**
     * Constructor for the SwapToLP class. Builds and returns a SwapToLP object.
     */
    private constructor();
    /**
     * Initializes a SwapToLP object.
     */
    static build(connection: Connection, user: PublicKey): Promise<SwapToLP>;
    /**
     * Generates a set of unsigned transactions for swapping a single SPL token to base/quote tokens required for LP. The transactions include setup transactions for the output tokens based on the LP token's base and quote mints.
     * @param inputTokenAmount
     * @param inputTokenMint
     * @param lpTokenMint
     */
    swap(inputTokenAmount: number, inputTokenMint: PublicKey, lpTokenMint: PublicKey): Promise<{
        transactions: any[];
        poolInfo: LiquidityPoolJsonInfo;
        prices: {
            inputTokenPrice: number;
            basePrice: number;
            quotePrice: number;
        };
        tokenInfo: {
            inputTokenInfo: any;
            baseInfo: any;
            quoteInfo: any;
        };
        baseRoute: RouteInfo;
        quoteRoute: RouteInfo;
    }>;
    /**
     * Generate an unsigned transactions with instruction to add liquidity to LP.
     * @param lpTokenMint
     * @param poolInfo
     * @param tokenAAmount
     */
    addLiquidity(lpTokenMint: PublicKey, poolInfo: LiquidityPoolJsonInfo, tokenAAmount: number, user: PublicKey, connection?: Connection): Promise<Transaction>;
    /**
     * Helper method to bundling `swapToBaseQuote` and `addLiquidity` together.
     * @param inputTokenAmount
     * @param inputTokenMint
     * @param lpTokenMint
     * @returns Array of transactions
     */
    swapAndAddLiquidity(inputTokenAmount: number, inputTokenMint: PublicKey, lpTokenMint: PublicKey, user: PublicKey, connection?: Connection): Promise<{
        transactions: any[];
        routes: {
            baseRoute: RouteInfo;
            quoteRoute: RouteInfo;
        };
        prices: {
            inputTokenPrice: number;
            basePrice: number;
            quotePrice: number;
        };
        tokenInfo: {
            inputTokenInfo: any;
            baseInfo: any;
            quoteInfo: any;
        };
    }>;
    /**
     * Get route info.
     * @param inputTokenAmount
     * @param inputTokenMint
     * @param lpTokenMint
     */
    getRouteInfo(inputTokenAmount: number, inputTokenMint: PublicKey, lpTokenMint: PublicKey): Promise<RouteInfo[]>;
    /**
     * Get pool info for LP token mint.
     * @param lpTokenMint - LP token mint
     */
    private getPoolInfo;
    /**
     * Get token info for a given mint.
     * @param tokenMint - token mint
     */
    private getTokenInfo;
    /**
     * Fetch user balances for 3 given mints.
     * @param tokenAMint
     * @param tokenBMint
     * @param lpTokenMint
     */
    getUserBalances(tokenAMint: PublicKey, tokenBMint: PublicKey, lpTokenMint: PublicKey, user: PublicKey): Promise<IUserBalances>;
    /**
     * Fetch user balances for an LP mint and its base and quote tokens.
     * @param lpTokenMint
     */
    getUserBalancesFromLp(lpTokenMint: PublicKey, user: PublicKey): Promise<IUserBalances>;
    /**
     * Get token price for a given mint.
     * @param tokenMint
     * @param tokenDecimals
     */
    getTokenPrice(tokenMint: string, tokenDecimals: number): Promise<number>;
}
export interface IUserBalances {
    tokenAAddress: PublicKey;
    tokenABalance: number;
    tokenBAddress: PublicKey;
    tokenBBalance: number;
    lpTokenAddress: PublicKey;
    lpTokenBalance: number;
}
