/// <reference types="bn.js" />
import { ConfirmOptions, Connection, PublicKey, Transaction, TransactionInstruction, Signer } from '@solana/web3.js';
import { BN, Idl, Program } from '@project-serum/anchor';
import { Token } from '@solana/spl-token';
/**
 * Constants
 */
export declare const STATE_SEED = "fstate1";
export declare const ACCOUNT_SEED = "faccv1";
export declare const CPI_SEED = "FABRIC";
/**
 * Protocol class for interacting with the Fabric protocol
 */
export declare class Protocol {
    wallet: IWallet;
    connection: Connection;
    programId: PublicKey;
    adminPubkey: PublicKey;
    assetsListPubkey: PublicKey;
    statePubkey: PublicKey;
    cpiAuthPubkey: PublicKey;
    idl: Idl;
    faucetIdl: Idl;
    usdcFaucetIdl: Idl;
    pythIdl: Idl;
    program: Program;
    faucetProgram: Program;
    usdcFaucetProgram: Program;
    state: any;
    assetsList: any;
    /**
     * @constructor
     * @param connection - web3.Connection instance
     * @param wallet - IWallet instance
     * @param programId - PublicKey of the program's program ID
     * @param faucetProgramId - PublicKey of the faucet program's program ID
     * @param opts - Confirmation options for provider
     */
    private constructor();
    /**
     * Initialises a Protocol object
     * @param connection - web3.Connection instance
     * @param wallet - IWallet instance
     * @param programId - PublicKey of the program's program ID
     * @param faucetProgramId - PublicKey of the faucet program's program ID
     * @param opts - Confirmation options for provider
     * @returns Procotol instance
     */
    static initialise(connection: Connection, wallet: IWallet, programId?: PublicKey, faucetProgramId?: PublicKey, usdcFaucetProgramId?: PublicKey, opts?: ConfirmOptions): Promise<Protocol>;
    /**
     * Initialises the contract and returns the transaction signature
     * @param epochDurationInSlots - The duration of an epoch in slots
     * @param rewardTokenAAccount - PublicKey of the reward token account
     * @param rewardWithdrawlDelay - The number of slots to wait before withdrawing rewards
     * @param rewardTokenAPerEpoch - The number of reward tokens to be issued per epoch
     * @returns Transaction signature
     */
    initialiseContract(epochDurationInSlots: number, rewardTokenAAccount: PublicKey, rewardWithdrawlDelay?: number, rewardTokenAPerEpoch?: number): Promise<string>;
    /**
     * Initialises a fUSD token and asset list
     * @param collateralMintPubkey
     * @param collateralFeedPubkey
     * @param collateralReserveAccountPubkey
     * @param collateralLiquidationFundAccountPubkey
     * @returns
     */
    initialiseAssetsList(collateralMintPubkey: PublicKey, collateralFeedPubkey: PublicKey, collateralReserveAccountPubkey: PublicKey, collateralLiquidationFundAccountPubkey: PublicKey): Promise<[string, Token, PublicKey]>;
    setAssetsListAccount(assetListPubkey: PublicKey): Promise<string>;
    depositCollateral(amount: BN, userAccount: PublicKey, collateralReserveAccount: PublicKey, userCollateralAccount: PublicKey, owner: PublicKey, signer?: Signer): Promise<string>;
    depositCollateralIx(amount: BN, userAccount: PublicKey, collateralReserveAccount: PublicKey, userCollateralAccount: PublicKey, owner: PublicKey): Promise<TransactionInstruction>;
    mint(amount: BN, owner: PublicKey, fusdReceiver: PublicKey, userAccount: PublicKey, signer?: Signer): Promise<string>;
    mintIx(amount: BN, owner: PublicKey, fusdReceiver: PublicKey, userAccount: PublicKey): Promise<TransactionInstruction>;
    burn(amount: BN, owner: PublicKey, fusdReceiver: PublicKey, userAccount: PublicKey, signer?: Signer): Promise<string>;
    burnIx(amount: BN, owner: PublicKey, fusdReceiver: PublicKey, userAccount: PublicKey): Promise<TransactionInstruction>;
    withdrawCollateral(amount: BN, userAccount: PublicKey, collateralReserveAccount: PublicKey, userCollateralAccount: PublicKey, owner: PublicKey, signer?: Signer): Promise<string>;
    withdrawCollateralIx(amount: BN, userAccount: PublicKey, collateralReserveAccount: PublicKey, userCollateralAccount: PublicKey, owner: PublicKey): Promise<TransactionInstruction>;
    swap(amount: BN, owner: PublicKey, tokenAMint: PublicKey, tokenBMint: PublicKey, tokenAReceiver: PublicKey, tokenBReceiver: PublicKey, userAccount: PublicKey, signer?: Signer): Promise<string>;
    swapIx(amount: BN, owner: PublicKey, tokenAMint: PublicKey, tokenBMint: PublicKey, tokenAReceiver: PublicKey, tokenBReceiver: PublicKey, userAccount: PublicKey): Promise<TransactionInstruction>;
    /**
     * Liquidates a user's account
     * @param amountInUSD - Amount of USD to liquidate
     * @param liquidator - Owner account of liquidator
     * @param fUSDMint - Mint of fUSD token
     * @param userToLiquidateAccount - User account of user to liquidate
     * @param collateralReserveAccount - Account of collateral reserve
     * @param collateralLiquidationFundAccount - Account of collateral liquidation fund
     * @param liquidatorsFusdAccount - The liquidator's fUSD account
     * @param liquidatorsCollateralAccount - The liquidator's collateral account
     * @returns Transaction signature
     */
    liquidate(amountInUSD: BN, liquidator: PublicKey, fUSDMint: PublicKey, userToLiquidateAccount: PublicKey, collateralReserveAccount: PublicKey, collateralLiquidationFundAccount: PublicKey, liquidatorsFusdAccount: PublicKey, liquidatorsCollateralAccount: PublicKey, signer?: Signer): Promise<string>;
    /**
     * Liquidates a user's account
     * @param amountInUSD - Amount of USD to liquidate
     * @param liquidator - Owner account of liquidator
     * @param fUSDMint - Mint of fUSD token
     * @param userToLiquidateAccount - User account of user to liquidate
     * @param collateralReserveAccount - Account of collateral reserve
     * @param collateralLiquidationFundAccount - Account of collateral liquidation fund
     * @param liquidatorsFusdAccount - The liquidator's fUSD account
     * @param liquidatorsCollateralAccount - The liquidator's collateral account
     * @returns Instruction
     */
    liquidateIx(amountInUSD: BN, liquidator: PublicKey, fUSDMint: PublicKey, userToLiquidateAccount: PublicKey, collateralReserveAccount: PublicKey, collateralLiquidationFundAccount: PublicKey, liquidatorsFusdAccount: PublicKey, liquidatorsCollateralAccount: PublicKey, signer?: Signer): Promise<TransactionInstruction>;
    claimRewards(userAccount: PublicKey): Promise<string>;
    claimRewardsIx(userAccount: PublicKey): Promise<TransactionInstruction>;
    withdrawRewards(userAccount: PublicKey, owner: PublicKey, rewardTokenAReceiver: PublicKey, signer?: Signer): Promise<string>;
    updatePrices(): Promise<string>;
    updatePricesIx(limit?: number): Promise<TransactionInstruction>;
    updateCollateralPricesIx(): Promise<TransactionInstruction>;
    updateSyntheticPricesIx(): Promise<TransactionInstruction>;
    updateUser(userAccount: PublicKey, signer?: Signer): Promise<string>;
    updateUserIx(userAccount: PublicKey): Promise<TransactionInstruction>;
    getStateAccount(): Promise<[PublicKey, number]>;
    getState(): Promise<IState>;
    /**
     * Subscribes to state changes
     * @param fn - Callback method
     */
    onStateChange(fn: (state: IState) => void): void;
    /**
     * Subscribes to user account changes
     * @param userAccount - PublicKey of the user account
     * @param fn - Callback method
     */
    onUserChange(userAccount: PublicKey, fn: (user: IUser) => void): void;
    /**
     * Subscribes to asset list changes
     * @param fn - Callback method
     */
    onAssetListChange(fn: (assetList: IAssetList) => void): void;
    /**
     * Fetch assets list.
     * @returns Assets list
     */
    getAssetList(): Promise<IAssetList>;
    /**
     * Create user account for protocol.
     * @param owner - Owner of the user account
     * @returns Address of user account
     */
    createUserAccount(owner: PublicKey): Promise<PublicKey>;
    /**
     * Create user account for protocol.
     * @param owner - Owner of the user account
     * @returns Instruction to create user account
     */
    createUserAccountIx(owner: PublicKey): Promise<TransactionInstruction>;
    /**
     * Get user PDA account and bump.
     * @param owner - Owner of the user account
     * @returns User PDA account and bump
     */
    getUserAccount(owner: PublicKey): Promise<[PublicKey, number]>;
    /**
     * Get user account.
     * @param userAccount - User account address
     * @returns User account
     */
    getUser(userAccount: PublicKey): Promise<IUser>;
    /**
     * Check if a user account already exists for an owner.
     * @param owner - PublicKey of the owner
     * @returns true if the user account exists, false otherwise
     */
    checkIfUserAccountExists(owner: PublicKey): Promise<boolean>;
    /**
     * Get CPI account address and bump.
     * @returns CPI account address and bump
     */
    getCpiAuthAccount(): Promise<[PublicKey, number]>;
    /**
     * Mint tokens to token account.
     * @param amount - Amount of tokens to mint
     * @param decimals - Amount scale
     * @param toAddress - Token account to mint tokens to
     * @param token - Token to mint
     * @param authority - Token authority
     * @param signer - Signer
     */
    mintTokens(amount: number, decimals: number, toAddress: PublicKey, token: Token, authority?: PublicKey, signer?: Signer): Promise<void>;
    /**
     * Creates a token account for an owner.
     * @param token - Token to create account for
     * @param owner - Owner
     * @returns Token account public key
     */
    createTokenAccount(token: Token, owner: PublicKey): Promise<PublicKey>;
    /**
     * Returns the balance for a given token account using uiAmount
     * @deprecated Use `getAccountBalance` instead
     * @param tokenAccount - PublicKey of user's token account
     * @returns User's token balance
     */
    getAccountBalanceForMint(tokenAccount: PublicKey): Promise<any>;
    /**
     * Returns the balance for a given token account
     * @param tokenAccount - PublicKey of user's token account
     * @returns User's token balance
     */
    getAccountBalance(tokenAccount: PublicKey): Promise<number>;
    /**
     * Returns the balance for a given token account
     * @param tokenAccount - PublicKey of user's token account
     * @returns User's token balance
     */
    getAccountBalanceScaled(tokenAccount: PublicKey): Promise<number>;
    /**
     * Returns a token account info
     * @param token - Token mint of associated account
     * @param tokenAccount - PublicKey of user's token account
     * @returns Token account info for user's associated account
     */
    getTokenAccountInfo(token: Token, tokenAccount: PublicKey): Promise<import("@solana/spl-token").AccountInfo>;
    /**
     * Returns a user's associated token account
     * @param mint - Token mint of associated account
     * @param owner - PublicKey of user's account
     * @returns PublicKey of user's associated account
     */
    getAssociatedTokenAccount(mint: PublicKey, owner: PublicKey): Promise<PublicKey>;
    /**
     * Fetches and returns the mint address of the fUSD synthetic token from state
     * @returns Returns the mint address of the fUSD synthetic token
     */
    getfUsdTokenMint(): Promise<PublicKey>;
    /**
     * Creates and returns a token mint
     * @param decimals - Number of decimals for token mint
     * @param authority - PublicKey of mint authority
     * @returns `Token` object
     */
    createToken(decimals: number, authority?: PublicKey): Promise<Token>;
    /**
     * Returns a user's C ratio
     * @param userAccount - PublicKey of user's token account
     * @returns C ratio
     */
    getUserCollateralRatio(userAccount: PublicKey, user?: IUser, state?: IState, assetList?: IAssetList): Promise<number>;
    /**
     * Returns the USD value of a user's debt
     * @param userAccount - PublicKey of user's account
     * @param user - User object
     * @param state - State object
     * @param assetList - Asset list object
     * @returns USD value of user's debt
     */
    getUserDebtUsd(userAccount: PublicKey, user?: IUser, state?: IState, assetList?: IAssetList): Promise<number>;
    /**
     * Returns the USD value of a user's debt scaled to 1e8
     * @param userAccount - PublicKey of user's account
     * @param user - User object
     * @param state - State object
     * @param assetList - Asset list object
     * @returns USD value of user's debt
     */
    getUserDebtUsdScaled(userAccount: PublicKey, user?: IUser, state?: IState, assetList?: IAssetList): Promise<number>;
    /**
     * Returns the USD value of a user's collateral
     * @param userAccount - PublicKey of user's token account
     * @param user - User object
     * @param assetList - Asset list object
     * @returns USD value of user's collateral
     */
    getUserCollateralUsd(userAccount: PublicKey, user?: IUser, assetList?: IAssetList): Promise<number>;
    /**
     * Returns the USD value of a user's collateral
     * @param userAccount - PublicKey of user's token account
     * @param user - User object
     * @param assetList - Asset list object
     * @returns USD value of user's collateral
     */
    getUserCollateralUsdScaled(userAccount: PublicKey, user?: IUser, assetList?: IAssetList): Promise<number>;
    /**
     * Returns the maximum possible user debt in USD
     * @param userAccount - PublicKey of user's account
     * @param user - User account object
     * @param assetList - Assset list object
     * @returns Maximum possible user debt in USD
     */
    getMaximumPossibleUserDebtUsd(userAccount: PublicKey, user?: IUser, assetList?: IAssetList): Promise<number>;
    /**
     * Returns the maximum possible user debt in USD
     * @param userAccount - PublicKey of user's account
     * @param user - User account object
     * @param assetList - Assset list object
     * @returns Maximum possible user debt in USD
     */
    getMaximumPossibleUserDebtUsdScaled(userAccount: PublicKey, user?: IUser, assetList?: IAssetList): Promise<number>;
    /**
     * Returns the global debt in USD
     * @param assetList - Asset list object
     * @returns Global debt in USD as BN scaled up to 1e8
     */
    getGlobalDebtUsd(assetList?: IAssetList): Promise<BN>;
    /**
     * Returns the supply and market cap for a specific synthetic
     * @param mint - PublicKey of synthetic's mint
     * @returns supply, scaled price and market cap
     */
    getSyntheticSupplyAndMarketCap(mint: PublicKey, assetList?: IAssetList, isFusd?: boolean): Promise<{
        supply: number;
        scaledPrice: number;
        marketCap: number;
    }>;
    /**
     * Returns the TVL and optimum C ratio for a specific collateral
     * @param mint - PublicKey of collateral's mint
     * @returns TVL and optimum C ratio
     */
    getCollateralTvlAndCRatio(mint: PublicKey, assetList?: IAssetList): Promise<{
        collateralTvl: number;
        collateralRatio: number;
    }>;
    /**
     * Halts the protocol and returns a transaction signature
     * @returns Transaction signature
     */
    haltProtocol(): Promise<string>;
    /**
     * Unhalts the protocol and returns a transaction signature
     * @returns Transaction signature
     */
    unhaltProtocol(): Promise<string>;
    /**
     * Updates the epoch duration and returns a transaction signature
     * @param durationInSlots
     * @returns Transaction signature
     */
    updateEpochDuration(durationInSlots: number): Promise<string>;
    /**
     * Updates the reward token A amount and returns a transaction signature
     * @param newRewardAmountScaled - New amount of reward token A per epoch
     * @returns Transaction signature
     */
    updateEpochRewardTokenAAmount(newRewardAmountScaled: BN): Promise<string>;
    /**
     * Updates the liqudation cycle duration in slots
     * @param newLiquidationCycleDuration - New duration for the liquidation cycle in slots
     * @returns Transaction signature
     */
    updateLiquidationCycleDuration(newLiquidationCycleDuration: number): Promise<string>;
    /**
     * Updates the reward token A account and returns a transaction signature
     * @param rewardTokenAAccount - New token A reward account
     * @returns Transaction signature
     */
    updateRewardTokenAAccount(rewardTokenAAccount: PublicKey): Promise<string>;
    /**
     * Returns transaction signature
     * @remarks This is a convenience method for calling `airdropFabDirectly`
     * @param owner - PublicKey of token account owner
     * @param amount - Amount to transfer (default of 100,000 FAB)
     * @returns Transaction signature
     */
    airdropFab(owner: PublicKey, amount?: number): Promise<string>;
    /**
     * Returns transaction signature
     * @remarks `owner` should own `fabAccount`
     * @param owner - PublicKey of token account owner
     * @param fabAccount - PublicKey of FAB token account
     * @param amount - Amount to transfer
     * @returns Transaction signature
     */
    airdropFabDirectly(owner: PublicKey, fabAccount: PublicKey, amount?: number): Promise<string>;
    /**
     * Returns transaction signature
     * @remarks This is a convenience method for calling `airdropFabDirectly`
     * @param owner - PublicKey of token account owner
     * @param amount - Amount to transfer (default of 100,000 USDC)
     * @returns Transaction signature
     */
    airdropUsdc(owner: PublicKey, amount?: number): Promise<string>;
    /**
     * Returns bump and address for FAB token mint
     * @remarks This method is only used on locally and on devnet.
     * @returns Bump and PublicKey for FAB token mint
     */
    getFabAirdropMintProgramAddress(): Promise<[PublicKey, number]>;
    /**
     * Returns bump and address for USDC token mint
     * @remarks This method is only used on locally and on devnet.
     * @returns Bump and PublicKey for USDC token mint
     */
    getUsdcAirdropMintProgramAddress(): Promise<[PublicKey, number]>;
    /**
     * Adds asset to asset list. Access restricted to admin only.
     * @param assetPriceFeed - Asset price feed
     * @param oracleType - Oracle type
     * @returns Transaction signature
     */
    addAsset(assetPriceFeed: PublicKey, oracleType: number): Promise<string>;
    /**
     * Adds synthetic to asset list. Access restricted to admin only.
     * @param assetMint - Asset mint address
     * @param assetPriceFeed - Asset price feed
     * @param maxSupply - Max supply
     * @returns Transaction signature
     */
    addSynthetic(assetMint: PublicKey, assetPriceFeed: PublicKey, maxSupply: number): Promise<string>;
    /**
     * Adds collateral to asset list. Access restricted to admin only.
     * @param assetMint - Asset mint address
     * @param assetPriceFeed - Asset price feed
     * @param collateralLiquidationFund - Collateral liquidation fund address
     * @param collateralReserve - Collateral reserve address
     * @param cRatio - Collateral ratio (i.e. for 1000%, 10 w/ scale 2 = 10*1e2 = 1000)
     * @param cRatioScale - Collateral ratio scale (i.e. 2)
     * @param supplyLimit - Supply limit
     * @param decimals - Mint decimals
     * @returns Transaction signature
     */
    addCollateral(assetMint: PublicKey, assetPriceFeed: PublicKey, collateralLiquidationFund: PublicKey, collateralReserve: PublicKey, cRatio: number, supplyLimit: number, decimals: number): Promise<string>;
    /**
     * Update an existing asset's price feed. Access restricted to admin only.
     * @param oldPriceFeed - Old price feed
     * @param newPriceFeed - New price feed
     * @param oracleType - Oracle type
     * @returns Transaction signature
     */
    updateAssetPriceFeed(oldPriceFeed: PublicKey, newPriceFeed: PublicKey, oracleType: number): Promise<string>;
    /**
     * Update a collateral's collateral ratio. Access restricted to admin only.
     * @param collateralMint - Collateral mint address
     * @param newCollateralRatio - New collateral ratio between 0 and 1
     * @returns Transaction signature
     */
    updateCollateralRatio(collateralMint: PublicKey, newCollateralRatio: number): Promise<string>;
    /**
     * Update a collateral's stakable limit. New limit must be larger than current staked. Access restricted to admin only.
     * @param collateralMint - Collateral mint address
     * @param newCollateralLimit - New collateral limit (unscaled)
     * @param newCollateralScale - New collateral limit scale
     * @returns Transaction signature
     */
    updateCollateralLimit(collateralMint: PublicKey, newCollateralLimit: number, newCollateralScale: number): Promise<string>;
    /**
     * Update synthetic swap fee. Access restricted to admin only.
     * @param syntheticMint - Mint address of synthetic
     * @param newSwapFee - New swap fee between 0 and 100
     * @returns Transaction signature
     */
    updateSyntheticSwapFee(syntheticMint: PublicKey, newSwapFee: number): Promise<string>;
    /**
     * Update synthetic's maximum supply. New supply must be greater than current supply. Access restricted to admin only.
     * @param syntheticMint - Mint address of synthetic
     * @param newSupply - New supply (unscaled) - will be scaled by 1e8
     * @returns Transaction signature
     */
    updateSyntheticMaximumSupply(syntheticMint: PublicKey, newSupply: number): Promise<string>;
    withdrawAccruedFees(toAccount: PublicKey): Promise<string>;
}
export interface IWallet {
    signTransaction(tx: Transaction): Promise<Transaction> | undefined;
    signAllTransactions(txs: Transaction[]): Promise<Transaction[]> | undefined;
    publicKey: PublicKey | null;
}
export interface IUser {
    owner: PublicKey;
    userDebtShares: BN;
    liquidationDeadline: BN;
    numberOfCollaterals: number;
    bump: number;
    userStakingData: IUserStakingData;
    userCollaterals: Array<IUserCollateral>;
    version: BN;
}
export interface IUserCollateral {
    amount: BN;
    collateralMintAddress: PublicKey;
    index: number;
}
export interface IUserStakingData {
    firstUpdate: BN;
    claimableRewards: IDecimal;
    lastEpochDebtShares: BN;
    currentEpochDebtShares: BN;
    nextEpochDebtShares: BN;
    lastUpdate: BN;
}
export interface IAsset {
    priceFeedAddress: PublicKey;
    price: IDecimal;
    lastUpdate: BN;
    status: number;
    oracleType: number;
}
export interface IAssetList {
    numberOfAssets: number;
    numberOfCollaterals: number;
    numberOfSynthetics: number;
    assets: Array<IAsset>;
    collaterals: Array<ICollateral>;
    synthetics: Array<ISynthetic>;
}
export interface IState {
    adminPubkey: PublicKey;
    cpiAuthPubkey: PublicKey;
    assetsList: PublicKey;
    nonce: number;
    bump: number;
    isHalted: boolean;
    liquidationData: ILiquidationData;
    delayBuffer: BN;
    rewardWithdrawlBuffer: BN;
    debtShares: BN;
    protocolSwapFee: IDecimal;
    accruedFees: IDecimal;
    globalStakingData: IGlobalStakingData;
}
export interface ILiquidationData {
    liquidationCycleDuration: BN;
    liquidationRate: IDecimal;
    liquidatorFee: IDecimal;
    protocolFee: IDecimal;
}
export interface ICollateral {
    assetIndex: number;
    collateralMintAddress: PublicKey;
    reserveAddress: PublicKey;
    liquidationFund: PublicKey;
    reserveBalance: IDecimal;
    collateralRatio: IDecimal;
    maximumCollateral: IDecimal;
    collateralMintBurnFee: IDecimal;
}
export interface ISynthetic {
    assetIndex: number;
    assetAddress: PublicKey;
    supply: IDecimal;
    maximumSupply: IDecimal;
    swapFeePercentage: IDecimal;
}
export interface IDecimal {
    val: BN;
    scale: number;
}
export interface IGlobalStakingData {
    epochDurationSlots: BN;
    rewardTokenAAccount: PublicKey;
    rewardTokenAPerEpoch: IDecimal;
    nextEpochData: IStakingEpochData;
    currentEpochData: IStakingEpochData;
    previousEpochData: IStakingEpochData;
}
export interface IStakingEpochData {
    globalDebtShares: BN;
    startSlot: BN;
    rewardTokenAForEpoch: IDecimal;
}
export declare enum OracleType {
    Fixed = 0,
    Pyth = 1,
    PythInverse = 2,
    Chainlink = 3,
    Switchboard = 4,
    Unknown = 5
}
