"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.OracleType = exports.Protocol = exports.CPI_SEED = exports.ACCOUNT_SEED = exports.STATE_SEED = void 0;
const web3_js_1 = require("@solana/web3.js");
const anchor_1 = require("@project-serum/anchor");
const fabric_synths_json_1 = __importDefault(require("./idl/fabric_synths.json"));
const fabric_faucet_json_1 = __importDefault(require("./idl/fabric_faucet.json"));
const fabric_faucet_json_2 = __importDefault(require("./idl/fabric_faucet.json"));
const spl_token_1 = require("@solana/spl-token");
/**
 * Constants
 */
exports.STATE_SEED = "fstate1";
exports.ACCOUNT_SEED = "faccv1";
exports.CPI_SEED = "FABRIC";
const PERCENTAGE_EXPONENT = 5;
const SUPPLY_EXPONENT = 8;
/**
 * Protocol class for interacting with the Fabric protocol
 */
class Protocol {
    /**
     * @constructor
     * @param connection - web3.Connection instance
     * @param wallet - IWallet instance
     * @param programId - PublicKey of the program's program ID
     * @param faucetProgramId - PublicKey of the faucet program's program ID
     * @param opts - Confirmation options for provider
     */
    constructor(connection, wallet, programId, faucetProgramId, usdcFaucetProgramId, opts) {
        this.idl = fabric_synths_json_1.default;
        this.faucetIdl = fabric_faucet_json_1.default;
        this.usdcFaucetIdl = fabric_faucet_json_2.default;
        this.wallet = wallet;
        this.programId = programId;
        this.connection = connection;
        const provider = new anchor_1.Provider(this.connection, this.wallet, opts || anchor_1.Provider.defaultOptions());
        this.program = new anchor_1.Program(fabric_synths_json_1.default, this.programId, provider);
        this.faucetProgram = new anchor_1.Program(fabric_faucet_json_1.default, faucetProgramId, provider);
        this.usdcFaucetProgram = new anchor_1.Program(fabric_faucet_json_2.default, usdcFaucetProgramId, provider);
    }
    /**
     * Initialises a Protocol object
     * @param connection - web3.Connection instance
     * @param wallet - IWallet instance
     * @param programId - PublicKey of the program's program ID
     * @param faucetProgramId - PublicKey of the faucet program's program ID
     * @param opts - Confirmation options for provider
     * @returns Procotol instance
     */
    static initialise(connection, wallet, programId = new web3_js_1.PublicKey('3PVSps2e8VGfWADi6YzwqdrcxYrF2nbwkneZxwXq9KMr'), faucetProgramId = new web3_js_1.PublicKey('3wX7rksfP6n6gJjURuPT5m7pQvPxmpw8G44AJmtY4Xgh'), usdcFaucetProgramId = new web3_js_1.PublicKey('BG4gXndsRBuqcHvZjZimcyA9YPMMhRqyfCcUMTuxpj88'), opts) {
        return __awaiter(this, void 0, void 0, function* () {
            var protocol = new Protocol(connection, wallet, programId, faucetProgramId, usdcFaucetProgramId, opts);
            protocol.statePubkey = (yield protocol.getStateAccount())[0];
            try {
                yield protocol.getState();
            }
            catch (_a) { }
            return protocol;
        });
    }
    /**
     * Initialises the contract and returns the transaction signature
     * @param epochDurationInSlots - The duration of an epoch in slots
     * @param rewardTokenAAccount - PublicKey of the reward token account
     * @param rewardWithdrawlDelay - The number of slots to wait before withdrawing rewards
     * @param rewardTokenAPerEpoch - The number of reward tokens to be issued per epoch
     * @returns Transaction signature
     */
    initialiseContract(epochDurationInSlots, rewardTokenAAccount, rewardWithdrawlDelay, rewardTokenAPerEpoch) {
        return __awaiter(this, void 0, void 0, function* () {
            var stateAddressInfo = yield this.getStateAccount();
            this.adminPubkey = this.wallet.publicKey;
            this.statePubkey = stateAddressInfo[0];
            let cpiAccountInfo = yield this.getCpiAuthAccount();
            this.cpiAuthPubkey = cpiAccountInfo[0];
            let epochDuration = new anchor_1.BN(epochDurationInSlots);
            var tx = yield this.program.rpc.initialiseContract(new anchor_1.BN(stateAddressInfo[1]), new anchor_1.BN(cpiAccountInfo[1]), epochDuration, new anchor_1.BN(rewardWithdrawlDelay || 0), new anchor_1.BN(rewardTokenAPerEpoch * 1e8 || 0), new anchor_1.BN(8), {
                accounts: {
                    stateAccount: this.statePubkey,
                    adminAccount: this.adminPubkey,
                    systemProgram: web3_js_1.SystemProgram.programId,
                    cpiAuthAccount: this.cpiAuthPubkey,
                    rewardTokenAAccount: rewardTokenAAccount
                }
            });
            return tx;
        });
    }
    /**
     * Initialises a fUSD token and asset list
     * @param collateralMintPubkey
     * @param collateralFeedPubkey
     * @param collateralReserveAccountPubkey
     * @param collateralLiquidationFundAccountPubkey
     * @returns
     */
    initialiseAssetsList(collateralMintPubkey, collateralFeedPubkey, collateralReserveAccountPubkey, collateralLiquidationFundAccountPubkey) {
        return __awaiter(this, void 0, void 0, function* () {
            const assetListAccount = web3_js_1.Keypair.generate();
            var stateAddressInfo = yield this.getStateAccount();
            let usdToken = yield this.createToken(SUPPLY_EXPONENT, this.cpiAuthPubkey);
            var accountTx = new web3_js_1.Transaction();
            const createAccountIx = yield this.program.account.assetsList.createInstruction(assetListAccount);
            accountTx.add(createAccountIx);
            accountTx.recentBlockhash = (yield this.connection.getRecentBlockhash()).blockhash;
            accountTx.feePayer = this.wallet.publicKey;
            accountTx.sign(assetListAccount);
            const signedCreateAccountTx = yield this.wallet.signTransaction(accountTx);
            const sig = yield this.connection.sendRawTransaction(signedCreateAccountTx.serialize());
            yield this.connection.confirmTransaction(sig, 'confirmed');
            let tx = yield this.program.rpc.initialiseAssetsList({
                accounts: {
                    stateAccount: stateAddressInfo[0],
                    assetsList: assetListAccount.publicKey,
                    adminAccount: this.wallet.publicKey,
                    collateralToken: collateralMintPubkey,
                    collateralTokenFeed: collateralFeedPubkey,
                    usdToken: usdToken.publicKey,
                    fabReserve: collateralReserveAccountPubkey,
                    fabLiquidationFund: collateralLiquidationFundAccountPubkey,
                    cpiAuthAccount: this.cpiAuthPubkey,
                    rent: web3_js_1.SYSVAR_RENT_PUBKEY
                },
                signers: [
                    // @ts-expect-error
                    this.program.provider.wallet.payer,
                ]
            });
            this.assetsListPubkey = assetListAccount.publicKey;
            let result = [
                tx,
                usdToken,
                assetListAccount.publicKey
            ];
            return result;
        });
    }
    setAssetsListAccount(assetListPubkey) {
        return __awaiter(this, void 0, void 0, function* () {
            let tx = yield this.program.rpc.setAssetsList({
                accounts: {
                    stateAccount: this.statePubkey,
                    adminAccount: this.adminPubkey,
                    assetsListAccount: assetListPubkey
                }
            });
            this.assetsListPubkey = assetListPubkey;
            return tx;
        });
    }
    depositCollateral(amount, userAccount, collateralReserveAccount, userCollateralAccount, owner, signer) {
        return __awaiter(this, void 0, void 0, function* () {
            var accounts = {
                accounts: {
                    stateAccount: this.statePubkey,
                    owner,
                    userAccount: userAccount,
                    userCollateralAccount,
                    tokenProgramAccount: spl_token_1.TOKEN_PROGRAM_ID,
                    collateralReserveAccount,
                    assetsListAccount: this.assetsListPubkey,
                    cpiAuthAccount: this.cpiAuthPubkey
                },
                instructions: [
                    (yield this.updatePricesIx())
                ],
                signers: [
                    // @ts-expect-error
                    signer || this.program.provider.wallet.payer,
                ]
            };
            let tx = yield this.program.rpc.deposit(amount, accounts);
            return tx;
        });
    }
    depositCollateralIx(amount, userAccount, collateralReserveAccount, userCollateralAccount, owner) {
        return __awaiter(this, void 0, void 0, function* () {
            var accounts = {
                accounts: {
                    stateAccount: this.statePubkey,
                    owner,
                    userAccount: userAccount,
                    userCollateralAccount,
                    tokenProgramAccount: spl_token_1.TOKEN_PROGRAM_ID,
                    collateralReserveAccount,
                    assetsListAccount: this.assetsListPubkey,
                    cpiAuthAccount: this.cpiAuthPubkey
                }
            };
            return (yield this.program.instruction.deposit(amount, accounts));
        });
    }
    mint(amount, owner, fusdReceiver, userAccount, signer) {
        return __awaiter(this, void 0, void 0, function* () {
            let tx = yield this.program.rpc.mint(amount, {
                accounts: {
                    stateAccount: this.statePubkey,
                    adminAccount: this.adminPubkey,
                    assetsListAccount: this.assetsListPubkey,
                    fusdTokenAccount: (yield this.getfUsdTokenMint()),
                    toAccount: fusdReceiver,
                    userAccount: userAccount,
                    tokenProgramAccount: spl_token_1.TOKEN_PROGRAM_ID,
                    owner,
                    cpiAuthAccount: this.cpiAuthPubkey
                },
                instructions: [
                    (yield this.updatePricesIx())
                ],
                signers: [
                    // @ts-expect-error
                    signer || this.program.provider.wallet.payer,
                ]
            });
            return tx;
        });
    }
    mintIx(amount, owner, fusdReceiver, userAccount) {
        return __awaiter(this, void 0, void 0, function* () {
            let ix = yield this.program.instruction.mint(amount, {
                accounts: {
                    stateAccount: this.statePubkey,
                    adminAccount: this.adminPubkey,
                    assetsListAccount: this.assetsListPubkey,
                    fusdTokenAccount: (yield this.getfUsdTokenMint()),
                    toAccount: fusdReceiver,
                    userAccount: userAccount,
                    tokenProgramAccount: spl_token_1.TOKEN_PROGRAM_ID,
                    owner,
                    cpiAuthAccount: this.cpiAuthPubkey
                }
            });
            return ix;
        });
    }
    burn(amount, owner, fusdReceiver, userAccount, signer) {
        return __awaiter(this, void 0, void 0, function* () {
            var tx = yield this.program.rpc.burn(amount, {
                accounts: {
                    stateAccount: this.statePubkey,
                    assetsListAccount: this.assetsListPubkey,
                    fusdTokenAccount: (yield this.getfUsdTokenMint()),
                    fusdToBurnAccount: fusdReceiver,
                    tokenProgramAccount: spl_token_1.TOKEN_PROGRAM_ID,
                    userAccount: userAccount,
                    owner: owner,
                },
                instructions: [
                    (yield this.updatePricesIx())
                ],
                signers: [
                    // @ts-expect-error
                    signer || this.program.provider.wallet.payer,
                ]
            });
            return tx;
        });
    }
    burnIx(amount, owner, fusdReceiver, userAccount) {
        return __awaiter(this, void 0, void 0, function* () {
            var ix = yield this.program.instruction.burn(amount, {
                accounts: {
                    stateAccount: this.statePubkey,
                    assetsListAccount: this.assetsListPubkey,
                    fusdTokenAccount: (yield this.getfUsdTokenMint()),
                    fusdToBurnAccount: fusdReceiver,
                    tokenProgramAccount: spl_token_1.TOKEN_PROGRAM_ID,
                    userAccount: userAccount,
                    owner: owner
                }
            });
            return ix;
        });
    }
    withdrawCollateral(amount, userAccount, collateralReserveAccount, userCollateralAccount, owner, signer) {
        return __awaiter(this, void 0, void 0, function* () {
            var tx = yield this.program.rpc.withdraw(amount, {
                accounts: {
                    stateAccount: this.statePubkey,
                    userAccount: userAccount,
                    collateralReserveAccount,
                    userCollateralAccount,
                    tokenProgramAccount: spl_token_1.TOKEN_PROGRAM_ID,
                    assetsListAccount: this.assetsListPubkey,
                    owner,
                    adminAccount: this.adminPubkey,
                    cpiAuthAccount: this.cpiAuthPubkey
                },
                instructions: [
                    (yield this.updatePricesIx())
                ],
                signers: [
                    // @ts-expect-error
                    signer || this.program.provider.wallet.payer,
                ]
            });
            return tx;
        });
    }
    withdrawCollateralIx(amount, userAccount, collateralReserveAccount, userCollateralAccount, owner) {
        return __awaiter(this, void 0, void 0, function* () {
            var ix = yield this.program.instruction.withdraw(amount, {
                accounts: {
                    stateAccount: this.statePubkey,
                    userAccount: userAccount,
                    collateralReserveAccount,
                    userCollateralAccount,
                    tokenProgramAccount: spl_token_1.TOKEN_PROGRAM_ID,
                    assetsListAccount: this.assetsListPubkey,
                    owner,
                    adminAccount: this.adminPubkey,
                    cpiAuthAccount: this.cpiAuthPubkey
                }
            });
            return ix;
        });
    }
    swap(amount, owner, tokenAMint, tokenBMint, tokenAReceiver, tokenBReceiver, userAccount, signer) {
        return __awaiter(this, void 0, void 0, function* () {
            var swapCtx = {
                accounts: {
                    stateAccount: this.statePubkey,
                    adminAccount: this.adminPubkey,
                    assetsListAccount: this.assetsListPubkey,
                    owner,
                    tokenProgramAccount: spl_token_1.TOKEN_PROGRAM_ID,
                    tokenInAccount: tokenAMint,
                    tokenOutAccount: tokenBMint,
                    userTokenInAccount: tokenAReceiver,
                    userTokenOutAccount: tokenBReceiver,
                    userAccount,
                    cpiAuthAccount: this.cpiAuthPubkey
                },
                signers: [
                    // @ts-expect-error
                    signer || this.program.provider.wallet.payer,
                ],
                instructions: [
                    (yield this.updatePricesIx())
                ]
            };
            var tx = yield this.program.rpc.swap(amount, swapCtx);
            return tx;
        });
    }
    swapIx(amount, owner, tokenAMint, tokenBMint, tokenAReceiver, tokenBReceiver, userAccount) {
        return __awaiter(this, void 0, void 0, function* () {
            var swapCtx = {
                accounts: {
                    stateAccount: this.statePubkey,
                    adminAccount: this.adminPubkey,
                    assetsListAccount: this.assetsListPubkey,
                    owner,
                    tokenProgramAccount: spl_token_1.TOKEN_PROGRAM_ID,
                    tokenInAccount: tokenAMint,
                    tokenOutAccount: tokenBMint,
                    userTokenInAccount: tokenAReceiver,
                    userTokenOutAccount: tokenBReceiver,
                    userAccount,
                    cpiAuthAccount: this.cpiAuthPubkey
                }
            };
            var ix = yield this.program.instruction.swap(amount, swapCtx);
            return ix;
        });
    }
    /**
     * Liquidates a user's account
     * @param amountInUSD - Amount of USD to liquidate
     * @param liquidator - Owner account of liquidator
     * @param fUSDMint - Mint of fUSD token
     * @param userToLiquidateAccount - User account of user to liquidate
     * @param collateralReserveAccount - Account of collateral reserve
     * @param collateralLiquidationFundAccount - Account of collateral liquidation fund
     * @param liquidatorsFusdAccount - The liquidator's fUSD account
     * @param liquidatorsCollateralAccount - The liquidator's collateral account
     * @returns Transaction signature
     */
    liquidate(amountInUSD, liquidator, fUSDMint, userToLiquidateAccount, collateralReserveAccount, collateralLiquidationFundAccount, liquidatorsFusdAccount, liquidatorsCollateralAccount, signer) {
        return __awaiter(this, void 0, void 0, function* () {
            var tx = yield this.program.rpc.liquidate(amountInUSD, {
                accounts: {
                    stateAccount: this.statePubkey,
                    assetsListAccount: this.assetsListPubkey,
                    cpiAuthAccount: this.cpiAuthPubkey,
                    tokenProgramAccount: spl_token_1.TOKEN_PROGRAM_ID,
                    collateralReserveAccount,
                    collateralLiquidationFundAccount,
                    fusdMintAccount: fUSDMint,
                    user: userToLiquidateAccount,
                    liquidator,
                    liquidatorsFusdAccount,
                    liquidatorsCollateralAccount
                },
                signers: [
                    // @ts-expect-error
                    signer || this.program.provider.wallet.payer,
                ],
                instructions: [
                    (yield this.updatePricesIx())
                ]
            });
            return tx;
        });
    }
    /**
     * Liquidates a user's account
     * @param amountInUSD - Amount of USD to liquidate
     * @param liquidator - Owner account of liquidator
     * @param fUSDMint - Mint of fUSD token
     * @param userToLiquidateAccount - User account of user to liquidate
     * @param collateralReserveAccount - Account of collateral reserve
     * @param collateralLiquidationFundAccount - Account of collateral liquidation fund
     * @param liquidatorsFusdAccount - The liquidator's fUSD account
     * @param liquidatorsCollateralAccount - The liquidator's collateral account
     * @returns Instruction
     */
    liquidateIx(amountInUSD, liquidator, fUSDMint, userToLiquidateAccount, collateralReserveAccount, collateralLiquidationFundAccount, liquidatorsFusdAccount, liquidatorsCollateralAccount, signer) {
        return __awaiter(this, void 0, void 0, function* () {
            var ix = yield this.program.instruction.liquidate(amountInUSD, {
                accounts: {
                    stateAccount: this.statePubkey,
                    assetsListAccount: this.assetsListPubkey,
                    cpiAuthAccount: this.cpiAuthPubkey,
                    tokenProgramAccount: spl_token_1.TOKEN_PROGRAM_ID,
                    collateralReserveAccount,
                    collateralLiquidationFundAccount,
                    fusdMintAccount: fUSDMint,
                    user: userToLiquidateAccount,
                    liquidator,
                    liquidatorsFusdAccount,
                    liquidatorsCollateralAccount
                },
                signers: [
                    // @ts-expect-error
                    signer || this.program.provider.wallet.payer,
                ]
            });
            return ix;
        });
    }
    claimRewards(userAccount) {
        return __awaiter(this, void 0, void 0, function* () {
            var tx = yield this.program.rpc.claimRewards({
                accounts: {
                    stateAccount: this.statePubkey,
                    adminAccount: this.adminPubkey,
                    userAccount
                }
            });
            return tx;
        });
    }
    claimRewardsIx(userAccount) {
        return __awaiter(this, void 0, void 0, function* () {
            var ix = yield this.program.instruction.claimRewards({
                accounts: {
                    stateAccount: this.statePubkey,
                    adminAccount: this.adminPubkey,
                    userAccount
                }
            });
            return ix;
        });
    }
    withdrawRewards(userAccount, owner, rewardTokenAReceiver, signer) {
        return __awaiter(this, void 0, void 0, function* () {
            var rewardTokenARewardsAccount = (yield this.getState()).globalStakingData.rewardTokenAAccount;
            var tx = yield this.program.rpc.withdrawRewards({
                accounts: {
                    stateAccount: this.statePubkey,
                    adminAccount: this.adminPubkey,
                    userAccount,
                    cpiAuthAccount: this.cpiAuthPubkey,
                    owner,
                    rewardTokenAAccount: rewardTokenARewardsAccount,
                    tokenProgramAccount: spl_token_1.TOKEN_PROGRAM_ID,
                    usersRewardTokenAAccount: rewardTokenAReceiver
                },
                signers: [
                    // @ts-expect-error
                    signer || this.program.provider.wallet.payer,
                ]
            });
            return tx;
        });
    }
    updatePrices() {
        return __awaiter(this, void 0, void 0, function* () {
            var assetList = yield this.getAssetList();
            var feedAddresses = assetList.assets
                .filter((asset) => !asset.priceFeedAddress.equals(new web3_js_1.PublicKey(0)))
                .map((asset) => {
                return {
                    pubkey: asset.priceFeedAddress,
                    isWritable: false,
                    isSigner: false
                };
            });
            var tx = yield this.program.rpc.setAssetPrices({
                accounts: {
                    assetsListAccount: this.assetsListPubkey,
                    state: this.statePubkey,
                },
                remainingAccounts: feedAddresses
            });
            return tx;
        });
    }
    updatePricesIx(limit = undefined) {
        return __awaiter(this, void 0, void 0, function* () {
            var assetList = yield this.getAssetList();
            var feedAddresses = assetList.assets
                .filter((asset) => !asset.priceFeedAddress.equals(new web3_js_1.PublicKey(0)))
                .map((asset) => {
                return {
                    pubkey: asset.priceFeedAddress,
                    isWritable: false,
                    isSigner: false
                };
            })
                .slice(0, limit || assetList.assets.length);
            var ix = yield this.program.instruction.setAssetPrices({
                accounts: {
                    assetsListAccount: this.assetsListPubkey,
                    state: this.statePubkey,
                },
                remainingAccounts: feedAddresses
            });
            return ix;
        });
    }
    updateCollateralPricesIx() {
        return __awaiter(this, void 0, void 0, function* () {
            var assetList = yield this.getAssetList();
            var collateralPriceFeeds = [];
            assetList.collaterals.forEach(element => {
                var asset = assetList.assets[element.assetIndex];
                if (!asset.priceFeedAddress.equals(new web3_js_1.PublicKey(0))) {
                    collateralPriceFeeds.push({
                        pubkey: asset.priceFeedAddress,
                        isWritable: false,
                        isSigner: false
                    });
                }
            });
            var ix = yield this.program.instruction.setAssetPrices({
                accounts: {
                    assetsListAccount: this.assetsListPubkey,
                    state: this.statePubkey,
                },
                remainingAccounts: collateralPriceFeeds
            });
            return ix;
        });
    }
    updateSyntheticPricesIx() {
        return __awaiter(this, void 0, void 0, function* () {
            var assetList = yield this.getAssetList();
            var collateralPriceFeeds = [];
            assetList.synthetics.forEach(element => {
                var asset = assetList.assets[element.assetIndex];
                if (!asset.priceFeedAddress.equals(new web3_js_1.PublicKey(0))) {
                    collateralPriceFeeds.push({
                        pubkey: asset.priceFeedAddress,
                        isWritable: false,
                        isSigner: false
                    });
                }
            });
            var ix = yield this.program.instruction.setAssetPrices({
                accounts: {
                    assetsListAccount: this.assetsListPubkey,
                    state: this.statePubkey,
                },
                remainingAccounts: collateralPriceFeeds
            });
            return ix;
        });
    }
    updateUser(userAccount, signer) {
        return __awaiter(this, void 0, void 0, function* () {
            var tx = yield this.program.rpc.updateUser({
                accounts: {
                    stateAccount: this.statePubkey,
                    cpiAuthAccount: this.cpiAuthPubkey,
                    assetsListAccount: this.assetsListPubkey,
                    userAccount
                },
                signers: [
                    // @ts-expect-error
                    signer || this.program.provider.wallet.payer,
                ]
            });
            return tx;
        });
    }
    updateUserIx(userAccount) {
        return __awaiter(this, void 0, void 0, function* () {
            var ix = yield this.program.instruction.updateUser({
                accounts: {
                    stateAccount: this.statePubkey,
                    cpiAuthAccount: this.cpiAuthPubkey,
                    assetsListAccount: this.assetsListPubkey,
                    userAccount
                }
            });
            return ix;
        });
    }
    // helper methods
    getStateAccount() {
        return __awaiter(this, void 0, void 0, function* () {
            const [stateAddress, stateBump] = yield web3_js_1.PublicKey.findProgramAddress([Buffer.from(anchor_1.utils.bytes.utf8.encode(exports.STATE_SEED))], this.program.programId);
            var state = [
                stateAddress,
                stateBump
            ];
            return state;
        });
    }
    ;
    getState() {
        return __awaiter(this, void 0, void 0, function* () {
            const state = yield this.program.account.state.fetch(this.statePubkey);
            this.adminPubkey = state.adminPubkey;
            this.assetsListPubkey = state.assetsList;
            this.cpiAuthPubkey = state.cpiAuthPubkey;
            return state;
        });
    }
    /**
     * Subscribes to state changes
     * @param fn - Callback method
     */
    onStateChange(fn) {
        this.program.account.state.subscribe(this.statePubkey).on('change', (state) => {
            fn(state);
        });
    }
    /**
     * Subscribes to user account changes
     * @param userAccount - PublicKey of the user account
     * @param fn - Callback method
     */
    onUserChange(userAccount, fn) {
        this.program.account.userAccount.subscribe(userAccount).on('change', (user) => {
            fn(user);
        });
    }
    /**
     * Subscribes to asset list changes
     * @param fn - Callback method
     */
    onAssetListChange(fn) {
        this.program.account.assetsList.subscribe(this.assetsListPubkey).on('change', (assetList) => {
            fn(assetList);
        });
    }
    /**
     * Fetch assets list.
     * @returns Assets list
     */
    getAssetList() {
        return __awaiter(this, void 0, void 0, function* () {
            const state = yield this.getState();
            const assetsList = yield this.program.account.assetsList.fetch(state.assetsList);
            assetsList.assets = assetsList.assets.slice(0, assetsList.numberOfAssets);
            assetsList.collaterals = assetsList.collaterals.slice(0, assetsList.numberOfCollaterals);
            assetsList.synthetics = assetsList.synthetics.slice(0, assetsList.numberOfSynthetics);
            return assetsList;
        });
    }
    /**
     * Create user account for protocol.
     * @param owner - Owner of the user account
     * @returns Address of user account
     */
    createUserAccount(owner) {
        return __awaiter(this, void 0, void 0, function* () {
            var [userAddress, userBump] = yield this.getUserAccount(owner);
            var tx = yield this.program.rpc.createUserAccount(userBump, {
                accounts: {
                    userAccount: userAddress,
                    adminAccount: owner,
                    payer: this.wallet.publicKey,
                    rent: web3_js_1.SYSVAR_RENT_PUBKEY,
                    systemProgram: web3_js_1.SystemProgram.programId,
                }
            });
            yield this.program.provider.connection.confirmTransaction(tx);
            return userAddress;
        });
    }
    /**
     * Create user account for protocol.
     * @param owner - Owner of the user account
     * @returns Instruction to create user account
     */
    createUserAccountIx(owner) {
        return __awaiter(this, void 0, void 0, function* () {
            var [userAddress, userBump] = yield this.getUserAccount(owner);
            var ix = yield this.program.instruction.createUserAccount(userBump, {
                accounts: {
                    userAccount: userAddress,
                    adminAccount: owner,
                    payer: this.wallet.publicKey,
                    rent: web3_js_1.SYSVAR_RENT_PUBKEY,
                    systemProgram: web3_js_1.SystemProgram.programId,
                }
            });
            return ix;
        });
    }
    /**
     * Get user PDA account and bump.
     * @param owner - Owner of the user account
     * @returns User PDA account and bump
     */
    getUserAccount(owner) {
        return __awaiter(this, void 0, void 0, function* () {
            const [account, bump] = yield web3_js_1.PublicKey.findProgramAddress([Buffer.from(anchor_1.utils.bytes.utf8.encode(exports.ACCOUNT_SEED)), owner.toBuffer()], this.program.programId);
            let result = [
                account,
                bump
            ];
            return result;
        });
    }
    /**
     * Get user account.
     * @param userAccount - User account address
     * @returns User account
     */
    getUser(userAccount) {
        return __awaiter(this, void 0, void 0, function* () {
            const user = yield this.program.account.userAccount.fetch(userAccount);
            return user;
        });
    }
    /**
     * Check if a user account already exists for an owner.
     * @param owner - PublicKey of the owner
     * @returns true if the user account exists, false otherwise
     */
    checkIfUserAccountExists(owner) {
        return __awaiter(this, void 0, void 0, function* () {
            const userAddress = yield this.getUserAccount(owner);
            try {
                yield this.getUser(userAddress[0]);
                return true;
            }
            catch (_a) {
                return false;
            }
        });
    }
    /**
     * Get CPI account address and bump.
     * @returns CPI account address and bump
     */
    getCpiAuthAccount() {
        return __awaiter(this, void 0, void 0, function* () {
            const [cpiAccount, cpiBump] = yield web3_js_1.PublicKey.findProgramAddress([
                Buffer.from(exports.CPI_SEED),
            ], this.programId);
            let result = [
                cpiAccount,
                cpiBump
            ];
            return result;
        });
    }
    /**
     * Mint tokens to token account.
     * @param amount - Amount of tokens to mint
     * @param decimals - Amount scale
     * @param toAddress - Token account to mint tokens to
     * @param token - Token to mint
     * @param authority - Token authority
     * @param signer - Signer
     */
    mintTokens(amount, decimals, toAddress, token, authority, signer) {
        return __awaiter(this, void 0, void 0, function* () {
            if (authority === undefined) {
                authority = this.wallet.publicKey;
            }
            var signers = [];
            if (signer !== undefined) {
                signers.push(signer);
            }
            // you must be the correct authority to mint tokens
            var amountToMint = new anchor_1.BN(amount * Math.pow(10, decimals));
            yield token.mintTo(toAddress, authority, signers, new spl_token_1.u64(amountToMint.toString()));
        });
    }
    /**
     * Creates a token account for an owner.
     * @param token - Token to create account for
     * @param owner - Owner
     * @returns Token account public key
     */
    createTokenAccount(token, owner) {
        return __awaiter(this, void 0, void 0, function* () {
            var address = yield token.createAccount(owner);
            return address;
        });
    }
    /**
     * Returns the balance for a given token account using uiAmount
     * @deprecated Use `getAccountBalance` instead
     * @param tokenAccount - PublicKey of user's token account
     * @returns User's token balance
     */
    getAccountBalanceForMint(tokenAccount) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            var accountInfo = yield this.connection.getParsedAccountInfo(tokenAccount);
            // @ts-expect-error
            var balance = (_a = accountInfo.value) === null || _a === void 0 ? void 0 : _a.data.parsed.info.tokenAmount.uiAmount;
            balance = (balance !== undefined) ? balance : 0;
            return balance;
        });
    }
    /**
     * Returns the balance for a given token account
     * @param tokenAccount - PublicKey of user's token account
     * @returns User's token balance
     */
    getAccountBalance(tokenAccount) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            var accountInfo = yield this.connection.getParsedAccountInfo(tokenAccount);
            // @ts-expect-error
            var amount = (_a = accountInfo.value) === null || _a === void 0 ? void 0 : _a.data.parsed.info.tokenAmount.amount;
            amount = (amount !== undefined) ? amount : 0;
            // @ts-expect-error
            var decimals = (_b = accountInfo.value) === null || _b === void 0 ? void 0 : _b.data.parsed.info.tokenAmount.decimals;
            decimals = (decimals !== undefined) ? decimals : 0;
            if (amount !== 0 && decimals !== 0) {
                var balance = amount / Math.pow(10, decimals);
                return balance;
            }
            return 0;
        });
    }
    /**
     * Returns the balance for a given token account
     * @param tokenAccount - PublicKey of user's token account
     * @returns User's token balance
     */
    getAccountBalanceScaled(tokenAccount) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            var accountInfo = yield this.connection.getParsedAccountInfo(tokenAccount);
            // @ts-expect-error
            var amount = (_a = accountInfo.value) === null || _a === void 0 ? void 0 : _a.data.parsed.info.tokenAmount.amount;
            amount = (amount !== undefined) ? amount : 0;
            return amount;
        });
    }
    /**
     * Returns a token account info
     * @param token - Token mint of associated account
     * @param tokenAccount - PublicKey of user's token account
     * @returns Token account info for user's associated account
     */
    getTokenAccountInfo(token, tokenAccount) {
        return __awaiter(this, void 0, void 0, function* () {
            const info = yield token.getAccountInfo(tokenAccount);
            return info;
        });
    }
    /**
     * Returns a user's associated token account
     * @param mint - Token mint of associated account
     * @param owner - PublicKey of user's account
     * @returns PublicKey of user's associated account
     */
    getAssociatedTokenAccount(mint, owner) {
        return __awaiter(this, void 0, void 0, function* () {
            var associatedAddress = yield spl_token_1.Token.getAssociatedTokenAddress(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, mint, owner);
            return associatedAddress;
        });
    }
    /**
     * Fetches and returns the mint address of the fUSD synthetic token from state
     * @returns Returns the mint address of the fUSD synthetic token
     */
    getfUsdTokenMint() {
        return __awaiter(this, void 0, void 0, function* () {
            var assetList = yield this.getAssetList();
            return assetList.synthetics[0].assetAddress;
        });
    }
    /**
     * Creates and returns a token mint
     * @param decimals - Number of decimals for token mint
     * @param authority - PublicKey of mint authority
     * @returns `Token` object
     */
    createToken(decimals, authority) {
        return __awaiter(this, void 0, void 0, function* () {
            if (authority === undefined) {
                authority = this.wallet.publicKey;
            }
            const token = yield spl_token_1.Token.createMint(this.connection, 
            // @ts-expect-error
            this.program.provider.wallet.payer, authority, null, decimals, spl_token_1.TOKEN_PROGRAM_ID);
            return token;
        });
    }
    /**
     * Returns a user's C ratio
     * @param userAccount - PublicKey of user's token account
     * @returns C ratio
     */
    getUserCollateralRatio(userAccount, user, state, assetList) {
        return __awaiter(this, void 0, void 0, function* () {
            const userDebtUsd = yield this.getUserDebtUsd(userAccount, user, state, assetList);
            const userCollateralUsd = yield this.getUserCollateralUsd(userAccount, user, assetList);
            if (userDebtUsd === 0 && userCollateralUsd === 0) {
                return 0;
            }
            if (userDebtUsd === 0 && userCollateralUsd !== 0) {
                return Infinity;
            }
            const userCollateralRatio = (userCollateralUsd / userDebtUsd);
            return userCollateralRatio;
        });
    }
    /**
     * Returns the USD value of a user's debt
     * @param userAccount - PublicKey of user's account
     * @param user - User object
     * @param state - State object
     * @param assetList - Asset list object
     * @returns USD value of user's debt
     */
    getUserDebtUsd(userAccount, user, state, assetList) {
        return __awaiter(this, void 0, void 0, function* () {
            var debt = yield this.getUserDebtUsdScaled(userAccount, user, state, assetList);
            return debt != 0 ? debt / 1e8 : 0;
        });
    }
    /**
     * Returns the USD value of a user's debt scaled to 1e8
     * @param userAccount - PublicKey of user's account
     * @param user - User object
     * @param state - State object
     * @param assetList - Asset list object
     * @returns USD value of user's debt
     */
    getUserDebtUsdScaled(userAccount, user, state, assetList) {
        return __awaiter(this, void 0, void 0, function* () {
            user = user || (yield this.getUser(userAccount));
            state = state || (yield this.getState());
            // fetch global usd scaled up
            const globalDebtUsd = yield this.getGlobalDebtUsd(assetList);
            const userDebtShares = user.userDebtShares;
            const globalDebtShares = state.debtShares;
            if (globalDebtShares.toNumber() === 0 || userDebtShares.toNumber() === 0) {
                return 0;
            }
            return globalDebtUsd.toNumber() * (userDebtShares.toNumber() / globalDebtShares.toNumber());
        });
    }
    /**
     * Returns the USD value of a user's collateral
     * @param userAccount - PublicKey of user's token account
     * @param user - User object
     * @param assetList - Asset list object
     * @returns USD value of user's collateral
     */
    getUserCollateralUsd(userAccount, user, assetList) {
        return __awaiter(this, void 0, void 0, function* () {
            var userCollateralUsd = yield this.getUserCollateralUsdScaled(userAccount, user, assetList);
            return userCollateralUsd != 0 ? userCollateralUsd / 1e8 : 0;
        });
    }
    /**
     * Returns the USD value of a user's collateral
     * @param userAccount - PublicKey of user's token account
     * @param user - User object
     * @param assetList - Asset list object
     * @returns USD value of user's collateral
     */
    getUserCollateralUsdScaled(userAccount, user, assetList) {
        return __awaiter(this, void 0, void 0, function* () {
            user = user || (yield this.getUser(userAccount));
            assetList = assetList || (yield this.getAssetList());
            const assets = assetList.assets;
            const collaterals = assetList.collaterals;
            const userCollaterals = user.userCollaterals;
            var sum = new anchor_1.BN(0);
            userCollaterals.forEach(userCollateral => {
                const collateral = collaterals[userCollateral.index];
                const asset = assets[collateral.assetIndex];
                var increment = userCollateral.amount
                    .div(new anchor_1.BN(10).pow(new anchor_1.BN(collateral.reserveBalance.scale)))
                    .mul(asset.price.val);
                sum = sum.add(increment);
            });
            return sum.toNumber();
        });
    }
    /**
     * Returns the maximum possible user debt in USD
     * @param userAccount - PublicKey of user's account
     * @param user - User account object
     * @param assetList - Assset list object
     * @returns Maximum possible user debt in USD
     */
    getMaximumPossibleUserDebtUsd(userAccount, user, assetList) {
        return __awaiter(this, void 0, void 0, function* () {
            var scaledMaxDebt = yield this.getMaximumPossibleUserDebtUsdScaled(userAccount, user, assetList);
            return scaledMaxDebt != 0 ? scaledMaxDebt / 1e8 : 0;
        });
    }
    /**
     * Returns the maximum possible user debt in USD
     * @param userAccount - PublicKey of user's account
     * @param user - User account object
     * @param assetList - Assset list object
     * @returns Maximum possible user debt in USD
     */
    getMaximumPossibleUserDebtUsdScaled(userAccount, user, assetList) {
        return __awaiter(this, void 0, void 0, function* () {
            user = user || (yield this.getUser(userAccount));
            assetList = assetList || (yield this.getAssetList());
            const assets = assetList.assets;
            const collaterals = assetList.collaterals;
            const userCollaterals = user.userCollaterals;
            var sum = new anchor_1.BN(0);
            userCollaterals.forEach(userCollateral => {
                const collateral = collaterals[userCollateral.index];
                const asset = assets[collateral.assetIndex];
                // calculate collateral value
                var collateralValue = asset.price.val
                    .mul(userCollateral.amount)
                    .div(new anchor_1.BN(10).pow(new anchor_1.BN(collateral.reserveBalance.scale)))
                    .mul(collateral.collateralRatio.val)
                    .div(new anchor_1.BN(10).pow(new anchor_1.BN(collateral.collateralRatio.scale)));
                sum = sum.add(collateralValue);
            });
            // validate sum is not less than 1
            if (sum.lt(new anchor_1.BN(1 * 1e8))) {
                return 0;
            }
            else {
                return sum.sub(new anchor_1.BN(1)).toNumber();
            }
        });
    }
    /**
     * Returns the global debt in USD
     * @param assetList - Asset list object
     * @returns Global debt in USD as BN scaled up to 1e8
     */
    getGlobalDebtUsd(assetList) {
        return __awaiter(this, void 0, void 0, function* () {
            assetList = assetList || (yield this.getAssetList());
            var sum = new anchor_1.BN(0);
            assetList.synthetics.forEach(element => {
                const asset = assetList.assets[element.assetIndex];
                sum = sum.add(element.supply.val
                    .mul(asset.price.val)
                    .div(new anchor_1.BN(10).pow(new anchor_1.BN(element.supply.scale))));
            });
            return sum;
        });
    }
    /**
     * Returns the supply and market cap for a specific synthetic
     * @param mint - PublicKey of synthetic's mint
     * @returns supply, scaled price and market cap
     */
    getSyntheticSupplyAndMarketCap(mint, assetList, isFusd = false) {
        return __awaiter(this, void 0, void 0, function* () {
            assetList = assetList || (yield this.getAssetList());
            const synthetic = assetList.synthetics.find(synth => synth.assetAddress.toBase58() === mint.toBase58());
            const asset = assetList.assets[synthetic.assetIndex];
            var supply = synthetic.supply.val.toNumber();
            supply = (supply === 0) ? 0 : supply / (Math.pow(10, synthetic.supply.scale));
            const price = asset.price.val.toNumber();
            const scaledPrice = price / (Math.pow(10, Math.abs(8)));
            const marketCap = supply * scaledPrice;
            return {
                supply,
                scaledPrice,
                marketCap
            };
        });
    }
    /**
     * Returns the TVL and optimum C ratio for a specific collateral
     * @param mint - PublicKey of collateral's mint
     * @returns TVL and optimum C ratio
     */
    getCollateralTvlAndCRatio(mint, assetList) {
        return __awaiter(this, void 0, void 0, function* () {
            assetList = assetList || (yield this.getAssetList());
            const collateral = assetList.collaterals.find(collateral => collateral.collateralMintAddress.toBase58() === mint.toBase58());
            const asset = assetList.assets[collateral.assetIndex];
            var collateralRatio = collateral.collateralRatio.val.toNumber();
            collateralRatio = (collateralRatio === 0) ? 0 : 1 / (collateralRatio / (Math.pow(10, collateral.collateralRatio.scale)));
            var tvl = collateral.reserveBalance.val.div(new anchor_1.BN(10).pow(new anchor_1.BN(collateral.reserveBalance.scale))).toNumber();
            tvl = (tvl === 0) ? 0 : tvl;
            const price = asset.price.val.toNumber();
            const scaledPrice = price / (Math.pow(10, 8));
            const collateralTvl = tvl * scaledPrice;
            return {
                collateralTvl,
                collateralRatio
            };
        });
    }
    /**
     * Halts the protocol and returns a transaction signature
     * @returns Transaction signature
     */
    haltProtocol() {
        return __awaiter(this, void 0, void 0, function* () {
            var tx = yield this.program.rpc.updateHalted(true, {
                accounts: {
                    stateAccount: this.statePubkey,
                    adminAccount: this.adminPubkey,
                }
            });
            return tx;
        });
    }
    /**
     * Unhalts the protocol and returns a transaction signature
     * @returns Transaction signature
     */
    unhaltProtocol() {
        return __awaiter(this, void 0, void 0, function* () {
            var tx = yield this.program.rpc.updateHalted(false, {
                accounts: {
                    stateAccount: this.statePubkey,
                    adminAccount: this.adminPubkey,
                }
            });
            return tx;
        });
    }
    /**
     * Updates the epoch duration and returns a transaction signature
     * @param durationInSlots
     * @returns Transaction signature
     */
    updateEpochDuration(durationInSlots) {
        return __awaiter(this, void 0, void 0, function* () {
            var tx = yield this.program.rpc.updateEpochDuration(new anchor_1.BN(durationInSlots), {
                accounts: {
                    stateAccount: this.statePubkey,
                    adminAccount: this.adminPubkey,
                }
            });
            return tx;
        });
    }
    /**
     * Updates the reward token A amount and returns a transaction signature
     * @param newRewardAmountScaled - New amount of reward token A per epoch
     * @returns Transaction signature
     */
    updateEpochRewardTokenAAmount(newRewardAmountScaled) {
        return __awaiter(this, void 0, void 0, function* () {
            var tx = yield this.program.rpc.updateEpochRewardAAmount(newRewardAmountScaled, {
                accounts: {
                    stateAccount: this.statePubkey,
                    adminAccount: this.adminPubkey,
                }
            });
            return tx;
        });
    }
    /**
     * Updates the liqudation cycle duration in slots
     * @param newLiquidationCycleDuration - New duration for the liquidation cycle in slots
     * @returns Transaction signature
     */
    updateLiquidationCycleDuration(newLiquidationCycleDuration) {
        return __awaiter(this, void 0, void 0, function* () {
            var tx = yield this.program.rpc.updateLiquidationCycleDuration(new anchor_1.BN(newLiquidationCycleDuration), {
                accounts: {
                    stateAccount: this.statePubkey,
                    adminAccount: this.adminPubkey,
                }
            });
            return tx;
        });
    }
    /**
     * Updates the reward token A account and returns a transaction signature
     * @param rewardTokenAAccount - New token A reward account
     * @returns Transaction signature
     */
    updateRewardTokenAAccount(rewardTokenAAccount) {
        return __awaiter(this, void 0, void 0, function* () {
            var tx = yield this.program.rpc.updateRewardTokenAAccount({
                accounts: {
                    newRewardAccount: rewardTokenAAccount,
                    stateAccount: this.statePubkey,
                    adminAccount: this.adminPubkey,
                    cpiAuthAccount: this.cpiAuthPubkey,
                }
            });
            return tx;
        });
    }
    /**
     * Returns transaction signature
     * @remarks This is a convenience method for calling `airdropFabDirectly`
     * @param owner - PublicKey of token account owner
     * @param amount - Amount to transfer (default of 100,000 FAB)
     * @returns Transaction signature
     */
    airdropFab(owner, amount = 100000) {
        return __awaiter(this, void 0, void 0, function* () {
            // find mint pda and bump
            const [pda, bump] = yield this.getFabAirdropMintProgramAddress();
            // find associated token account
            const associatedTokenAccount = yield spl_token_1.Token.getAssociatedTokenAddress(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, pda, owner, true);
            const airdropAmount = new anchor_1.BN(amount * 1e8);
            var tx = yield this.faucetProgram.rpc.airdrop(bump, airdropAmount, {
                accounts: {
                    mint: pda,
                    receiver: owner,
                    destination: associatedTokenAccount,
                    rent: web3_js_1.SYSVAR_RENT_PUBKEY,
                    tokenProgram: spl_token_1.TOKEN_PROGRAM_ID,
                    systemProgram: web3_js_1.SystemProgram.programId,
                    associatedTokenProgram: spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID,
                    payer: this.wallet.publicKey
                }
            });
            return tx;
        });
    }
    /**
     * Returns transaction signature
     * @remarks `owner` should own `fabAccount`
     * @param owner - PublicKey of token account owner
     * @param fabAccount - PublicKey of FAB token account
     * @param amount - Amount to transfer
     * @returns Transaction signature
     */
    airdropFabDirectly(owner, fabAccount, amount = 100000) {
        return __awaiter(this, void 0, void 0, function* () {
            // find mint pda and bump
            const [pda, bump] = yield this.getFabAirdropMintProgramAddress();
            const airdropAmount = new anchor_1.BN(amount * 1e8);
            var tx = yield this.faucetProgram.rpc.airdrop(bump, airdropAmount, {
                accounts: {
                    mint: pda,
                    receiver: owner,
                    destination: fabAccount,
                    rent: web3_js_1.SYSVAR_RENT_PUBKEY,
                    tokenProgram: spl_token_1.TOKEN_PROGRAM_ID,
                    systemProgram: web3_js_1.SystemProgram.programId,
                    associatedTokenProgram: spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID,
                    payer: this.wallet.publicKey
                }
            });
            return tx;
        });
    }
    /**
     * Returns transaction signature
     * @remarks This is a convenience method for calling `airdropFabDirectly`
     * @param owner - PublicKey of token account owner
     * @param amount - Amount to transfer (default of 100,000 USDC)
     * @returns Transaction signature
     */
    airdropUsdc(owner, amount = 1000) {
        return __awaiter(this, void 0, void 0, function* () {
            // find mint pda and bump
            const [pda, bump] = yield this.getUsdcAirdropMintProgramAddress();
            // find associated token account
            const associatedTokenAccount = yield spl_token_1.Token.getAssociatedTokenAddress(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, pda, owner, true);
            const airdropAmount = new anchor_1.BN(amount * 1e8);
            var tx = yield this.usdcFaucetProgram.rpc.airdrop(bump, airdropAmount, {
                accounts: {
                    mint: pda,
                    receiver: owner,
                    destination: associatedTokenAccount,
                    rent: web3_js_1.SYSVAR_RENT_PUBKEY,
                    tokenProgram: spl_token_1.TOKEN_PROGRAM_ID,
                    systemProgram: web3_js_1.SystemProgram.programId,
                    associatedTokenProgram: spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID,
                    payer: this.wallet.publicKey
                }
            });
            return tx;
        });
    }
    /**
     * Returns bump and address for FAB token mint
     * @remarks This method is only used on locally and on devnet.
     * @returns Bump and PublicKey for FAB token mint
     */
    getFabAirdropMintProgramAddress() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield web3_js_1.PublicKey.findProgramAddress([Buffer.from(anchor_1.utils.bytes.utf8.encode("FAB-FAUCET"))], this.faucetProgram.programId);
        });
    }
    /**
     * Returns bump and address for USDC token mint
     * @remarks This method is only used on locally and on devnet.
     * @returns Bump and PublicKey for USDC token mint
     */
    getUsdcAirdropMintProgramAddress() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield web3_js_1.PublicKey.findProgramAddress([Buffer.from(anchor_1.utils.bytes.utf8.encode("USDC-FAUCET"))], this.usdcFaucetProgram.programId);
        });
    }
    /**
     * Adds asset to asset list. Access restricted to admin only.
     * @param assetPriceFeed - Asset price feed
     * @param oracleType - Oracle type
     * @returns Transaction signature
     */
    addAsset(assetPriceFeed, oracleType) {
        return __awaiter(this, void 0, void 0, function* () {
            var tx = yield this.program.rpc.addAsset(assetPriceFeed, // oracle price feed address
            oracleType, // oracle type
            {
                accounts: {
                    stateAccount: this.statePubkey,
                    adminAccount: this.adminPubkey,
                    assetsListAccount: this.assetsListPubkey,
                }
            });
            return tx;
        });
    }
    /**
     * Adds synthetic to asset list. Access restricted to admin only.
     * @param assetMint - Asset mint address
     * @param assetPriceFeed - Asset price feed
     * @param maxSupply - Max supply
     * @returns Transaction signature
     */
    addSynthetic(assetMint, assetPriceFeed, maxSupply) {
        return __awaiter(this, void 0, void 0, function* () {
            var tx = yield this.program.rpc.addSynthetic(new anchor_1.BN(maxSupply), // max supply
            {
                accounts: {
                    stateAccount: this.statePubkey,
                    adminAccount: this.adminPubkey,
                    assetsListAccount: this.assetsListPubkey,
                    assetMintAccount: assetMint,
                    priceFeedAddress: assetPriceFeed,
                }
            });
            return tx;
        });
    }
    /**
     * Adds collateral to asset list. Access restricted to admin only.
     * @param assetMint - Asset mint address
     * @param assetPriceFeed - Asset price feed
     * @param collateralLiquidationFund - Collateral liquidation fund address
     * @param collateralReserve - Collateral reserve address
     * @param cRatio - Collateral ratio (i.e. for 1000%, 10 w/ scale 2 = 10*1e2 = 1000)
     * @param cRatioScale - Collateral ratio scale (i.e. 2)
     * @param supplyLimit - Supply limit
     * @param decimals - Mint decimals
     * @returns Transaction signature
     */
    addCollateral(assetMint, assetPriceFeed, collateralLiquidationFund, collateralReserve, cRatio, supplyLimit, decimals) {
        return __awaiter(this, void 0, void 0, function* () {
            var tx = yield this.program.rpc.addCollateral(new anchor_1.BN(cRatio), new anchor_1.BN(PERCENTAGE_EXPONENT), new anchor_1.BN(supplyLimit), new anchor_1.BN(decimals), {
                accounts: {
                    stateAccount: this.statePubkey,
                    admin: this.adminPubkey,
                    assetsList: this.assetsListPubkey,
                    assetMint,
                    priceFeedAddress: assetPriceFeed,
                    collateralLiquidationFund,
                    collateralReserve,
                    cpiAuth: this.cpiAuthPubkey,
                }
            });
            return tx;
        });
    }
    /**
     * Update an existing asset's price feed. Access restricted to admin only.
     * @param oldPriceFeed - Old price feed
     * @param newPriceFeed - New price feed
     * @param oracleType - Oracle type
     * @returns Transaction signature
     */
    updateAssetPriceFeed(oldPriceFeed, newPriceFeed, oracleType) {
        return __awaiter(this, void 0, void 0, function* () {
            var tx = yield this.program.rpc.updateAssetPriceFeed(oracleType, {
                accounts: {
                    assetsList: this.assetsListPubkey,
                    admin: this.adminPubkey,
                    assetOldPriceFeed: oldPriceFeed,
                    assetNewPriceFeed: newPriceFeed,
                    stateAccount: this.statePubkey,
                }
            });
            return tx;
        });
    }
    /**
     * Update a collateral's collateral ratio. Access restricted to admin only.
     * @param collateralMint - Collateral mint address
     * @param newCollateralRatio - New collateral ratio between 0 and 1
     * @returns Transaction signature
     */
    updateCollateralRatio(collateralMint, newCollateralRatio) {
        return __awaiter(this, void 0, void 0, function* () {
            if (newCollateralRatio > 1) {
                throw new Error("Collateral ratio must be less than 1");
            }
            var tx = yield this.program.rpc.updateCollateralRatio(new anchor_1.BN(newCollateralRatio * (Math.pow(10, PERCENTAGE_EXPONENT))), new anchor_1.BN(PERCENTAGE_EXPONENT), {
                accounts: {
                    assetsList: this.assetsListPubkey,
                    admin: this.adminPubkey,
                    collateralMint,
                    stateAccount: this.statePubkey,
                }
            });
            return tx;
        });
    }
    /**
     * Update a collateral's stakable limit. New limit must be larger than current staked. Access restricted to admin only.
     * @param collateralMint - Collateral mint address
     * @param newCollateralLimit - New collateral limit (unscaled)
     * @param newCollateralScale - New collateral limit scale
     * @returns Transaction signature
     */
    updateCollateralLimit(collateralMint, newCollateralLimit, newCollateralScale) {
        return __awaiter(this, void 0, void 0, function* () {
            var tx = yield this.program.rpc.updateMaximumCollateral(new anchor_1.BN(newCollateralLimit * (Math.pow(10, newCollateralScale))), new anchor_1.BN(newCollateralScale), {
                accounts: {
                    assetsList: this.assetsListPubkey,
                    admin: this.adminPubkey,
                    collateralMint,
                    stateAccount: this.statePubkey,
                }
            });
            return tx;
        });
    }
    /**
     * Update synthetic swap fee. Access restricted to admin only.
     * @param syntheticMint - Mint address of synthetic
     * @param newSwapFee - New swap fee between 0 and 100
     * @returns Transaction signature
     */
    updateSyntheticSwapFee(syntheticMint, newSwapFee) {
        return __awaiter(this, void 0, void 0, function* () {
            var tx = yield this.program.rpc.updateSyntheticSwapFee(new anchor_1.BN((newSwapFee / 100) * (Math.pow(10, PERCENTAGE_EXPONENT))), new anchor_1.BN(PERCENTAGE_EXPONENT), {
                accounts: {
                    assetsList: this.assetsListPubkey,
                    admin: this.adminPubkey,
                    syntheticMint,
                    stateAccount: this.statePubkey,
                }
            });
            return tx;
        });
    }
    /**
     * Update synthetic's maximum supply. New supply must be greater than current supply. Access restricted to admin only.
     * @param syntheticMint - Mint address of synthetic
     * @param newSupply - New supply (unscaled) - will be scaled by 1e8
     * @returns Transaction signature
     */
    updateSyntheticMaximumSupply(syntheticMint, newSupply) {
        return __awaiter(this, void 0, void 0, function* () {
            var tx = yield this.program.rpc.updateSyntheticMaximumSupply(new anchor_1.BN(newSupply), new anchor_1.BN(SUPPLY_EXPONENT), {
                accounts: {
                    assetsList: this.assetsListPubkey,
                    admin: this.adminPubkey,
                    syntheticMint,
                    stateAccount: this.statePubkey,
                }
            });
            return tx;
        });
    }
    withdrawAccruedFees(toAccount) {
        return __awaiter(this, void 0, void 0, function* () {
            var fUSDMint = (yield this.getAssetList()).synthetics[0].assetAddress;
            var tx = yield this.program.rpc.withdrawAccruedFees({
                accounts: {
                    stateAccount: this.statePubkey,
                    adminAccount: this.adminPubkey,
                    cpiAuthAccount: this.cpiAuthPubkey,
                    fusdTokenAccount: fUSDMint,
                    tokenProgramAccount: spl_token_1.TOKEN_PROGRAM_ID,
                    toAccount,
                    assetsListAccount: this.assetsListPubkey,
                }
            });
            return tx;
        });
    }
}
exports.Protocol = Protocol;
;
;
var OracleType;
(function (OracleType) {
    OracleType[OracleType["Fixed"] = 0] = "Fixed";
    OracleType[OracleType["Pyth"] = 1] = "Pyth";
    OracleType[OracleType["PythInverse"] = 2] = "PythInverse";
    OracleType[OracleType["Chainlink"] = 3] = "Chainlink";
    OracleType[OracleType["Switchboard"] = 4] = "Switchboard";
    OracleType[OracleType["Unknown"] = 5] = "Unknown";
})(OracleType = exports.OracleType || (exports.OracleType = {}));
